 /* File:   lcd-6.c
  * Author: BILL
  *
  * Created on June 26 2016 11:00  AM
  *
  * ===================== TO BE REDONE FOR PIC16F877A =====================
  * WRITTEN TO RUN WITH PIC16F877A WITH 4 MEG CRYSTAL
  *
  * .
  * THE PIC MONITORS THE INPUTS AND DISPLAYS FAULT MESSAGES ON THE LCD SCREEN.
  * THE LCD IS A 2 X 16 WITH BACKLIGHT.
  * IT IS RUNNING IN 4 BIT MODE TAKING DATA FROM RB0, RB1, RB2, RB3.
  * ALL INPUTS ARE TIED HIGH BY 10K RESISTORS
  * ALL INPUTS ARE VIA OPTO COUPLERS.
  *
  * =================== END  TO BE REDONE FOR PIC16F877A =====================
  *
  *                     1 1 1 1 1 1 1
  *   1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6
  * +-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o----------------------------------------+
  * | +---------------------------------------------------------------------+ |
  * | |                                                                     | |
  * | |                                                                     | |
  * | |                HD44780 RUNNING IN 4 BIT MODE.                       | |
  * | |                                                                     | |
  * | |                                                                     | |
  * | +---------------------------------------------------------------------+ |
  * +------------------------------------------------------------------------+
  *
  *  PIN
  *   1 = VSS GND      --------------|
  *   2 = VDD +5V      --/\/\/\/\/\--|  5k preset pot
  *   3 = VEE Contrast ------^
  *   4 = RS <= RB4
  *   5 = R/W GROUNDED
  *   6 = E <= RB5
  *   7 = D0 NC
  *   8 = D1 NC
  *   9 = D2 NC
  *  10 = D3 NC
  *  11 = D4 <= RB0
  *  12 = D5 <= RB1
  *  13 = D6 <= RB2
  *  14 = D7 <= RB3
  *  15 = LED + TAKEN FROM SUPPLY + VIA RESISTORS
  *  16 = LED - TAKEN FROM SUPPLY -
  *
  *
  */ 

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#include "lcd.h"
#include "psu_msg.h"


// BEGIN CONFIG
#pragma config FOSC = HS   // Oscillator Selection bits (HS oscillator)
#pragma config WDTE = OFF  // Watchdog Timer Enable bit (WDT enabled)
#pragma config PWRTE = OFF // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOREN = ON  // Brown-out Reset Enable bit (BOR enabled)
#pragma config LVP = OFF   // Low-Voltage (Single-Supply) In-Circuit Serial Programming Enable bit (RB3 is digital I/O, HV on MCLR must be used for programming)
#pragma config CPD = OFF   // Data EEPROM Memory Code Protection bit (Data EEPROM code protection off)
#pragma config WRT = OFF   // Flash Program Memory Write Enable bits (Write protection off; all program memory may be written to by EECON control)
#pragma config CP = OFF    // Flash Program Memory Code Protection bit (Code protection off)
//END CONFIG

// DEFINING C PORT FOR MAIN INPUT
#define FAULT_0 RC0 // input port
#define FAULT_1 RC1 // input port
#define FAULT_2 RC2 // input port
#define FAULT_3 RC3 // input port
#define FAULT_4 RC4 // input port
#define FAULT_5 RC5 // input port
#define FAULT_6 RC6 // input port
#define FAULT_7 RC7 // input port
//END PORT C DEFINITIONS



// ************************ ADC BEGINS HERE. *******************************
// set MAXVOLTAGE to the maximum value of an A2D conversion.
#define MAXVOLTAGE 5.0  // prescale here
#define ADCMAX (1U<<10) // One greater than max. count for 10 bit ADC
void init(void)
{
    //	lcd_init(FOURBIT_MODE);
    PIE1bits.ADIE=0;	// not interrupt driven
    ADCON1bits.PCFG=0b1110; // Only AN0 as analog
    ADCON0bits.ADCS0=1;  // ADCS=0b001, Tad=8*Tosc (2us @Fosc=4MHz)
    ADCON0bits.ADCS1=0;
    ADCON1bits.ADCS2=0;
    ADCON1bits.ADFM=1;  // Right Justified
    ADCON0bits.ADON=1;	// enable A2D converter
}

unsigned int ReadADC(unsigned char n)
{
    ADCON0bits.CHS=n;  // Select channel n
    DelayUs(20); // ADC acquisition time
    ADCON0bits.GO=1;
    while(ADCON0bits.GO)continue; // Wait for conversion
    PIR1bits.ADIF=0;
    return(256U*ADRESH|ADRESL); // get result
}
//******************************* END ADC *********************************

//*************************************************************************
 void main(void)     //  MAIN PROCESSING HERE  //
//*************************************************************************
{
//*************** ADDED FOR ADC ******************
    unsigned int adres, last_value;
    unsigned int volts, decivolts;
    unsigned char outString[20];
//****************END OF ADDED FOR ADC ************
    unsigned int a;
    //     MSB-----LSB
    TRISA =    0b00011;   // UNUSED A/D TO BE SET AS REQUIRED
    TRISB = 0b00000000;   // LCD DATA AT B0 - B3   ICSP AT B6 & B7
    TRISC = 0b11111111;   // EIGHT INPUTS
    TRISD = 0b00000000;   // currently set to all output
    TRISE =      0b000;   // UNUSED A/D TO BE SET AS REQUIRED
    
    Lcd_Init();
    FAULT_0 = 1;        // MAKE ALL INPUTS HIGH
    FAULT_1 = 1;
    FAULT_2 = 1;
    FAULT_3 = 1;
    FAULT_4 = 1;
    FAULT_5 = 1;
    FAULT_6 = 1;
    FAULT_7 = 1;

    int HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                        // POWER UP OR RESET IF NO FAULTS ARE FOUND.
 while(1)
  {
  //*************************************************************************
    if (FAULT_0==0)                   // FAULT, RC0 AT 0V
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position
            Lcd_Write_String (MSG_0A);
            Lcd_Set_Cursor(2,1);                 // Select second line
            Lcd_Write_String (MSG_0B);
            FAULT_0 = 1;
            HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                            // POWER UP OR RESET IF NO FAULTS ARE ACTIVATED.
             while(1);
        }
//*************************************************************************
    else if (FAULT_1==0)                // FAULT, RC1 AT 0V
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position
            Lcd_Write_String (MSG_1A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String (MSG_1B);
            FAULT_1 = 1;
            HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                            // POWER UP OR RESET IF NO FAULTS ARE ACTIVATED.
            while(1);
        }
//*************************************************************************    
    else if (FAULT_2==0)                //FAULT, RC2 AT 0V
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position
            Lcd_Write_String (MSG_2A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String (MSG_2B);
            FAULT_2 = 1;
            HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                            // POWER UP OR RESET IF NO FAULTS ARE ACTIVATED.
            while(1);
        }
//*************************************************************************
    else if (FAULT_3==0)                // FAULT, RC3 AT 0V
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position
            Lcd_Write_String (MSG_3A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String (MSG_3B);
            FAULT_3 = 1;
            HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                            // POWER UP OR RESET IF NO FAULTS ARE ACTIVATED.
            while(1);
        }
//*************************************************************************
    else if (FAULT_4==0)                // FAULT, RC43 AT 0V
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position)
            Lcd_Write_String (MSG_4A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String (MSG_4B);
            FAULT_4 = 1;
            HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                            // POWER UP OR RESET IF NO FAULTS ARE ACTIVATED.
            while(1);
        }
//*************************************************************************
    else if (FAULT_5==0)                    //FAULT, RC5 AT 0V
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position
            Lcd_Write_String (MSG_5A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String (MSG_5B);
            FAULT_5 = 1;
            HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                            // POWER UP OR RESET IF NO FAULTS ARE ACTIVATED.
            while(1);
        }
//*************************************************************************
    else if (FAULT_6==0)                // FAULT, RC6 AT 0V
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);     // select first line first char position)
            Lcd_Write_String (MSG_6A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String (MSG_6B);
            FAULT_6 = 1;
            HADFAULT = 1;   // TO FORCE THE MAIN SCREEN DISPLAY TO COME ON AT
                            // POWER UP OR RESET IF NO FAULTS ARE ACTIVATED.
            while(1);
        }
//************************************************************************

//************************************************************************
//********************   START OF ANALOG INPUTS.  ************************
//***********************  ANALOGE VOLT METERS    ************************
else if (FAULT_7==0)
    {
        Lcd_Init();               // initialize LCD
        Lcd_Clear();              // clear LCD
        init();
        Lcd_Set_Cursor(1,1);
        Lcd_Write_String("Adjust the");
        Lcd_Set_Cursor(2,1);	// select line 2
        Lcd_Write_String("potentiometer");
        DelayMs(1000);
        last_value=-1; // Force update on first pass
        while(1)
            {
                // READ ADC(0) AND DISPLAY ON LINE ONE.
                adres=ReadADC(0);
                if(adres!=last_value)
            {
                volts=( // scale to voltage as fixed point, 2DP, with rounding.
                adres*(unsigned long)(MAXVOLTAGE*100) + ADCMAX/2)/ADCMAX;
                decivolts = volts%100;  // extract decimals
                volts = volts/100;      //extract integer volts
                Lcd_Clear();
                //	sprintf(outString,"ANO: %3.3Xh, %u.%2.2uV",adres,volts,decivolts);
                sprintf(outString,"V+ PSU = +%u.%2.2uV",volts,decivolts);
                Lcd_Write_String(outString);
                last_value = adres;
            }
            // READ ADC(1) AND DISPLAY ON LINE TWO.
            adres=ReadADC(1);
            //   if(adres!=last_value)
            {
                volts=( // scale to voltage as fixed point, 2DP, with rounding.
                adres*(unsigned long)(MAXVOLTAGE*100) + ADCMAX/2)/ADCMAX;
                decivolts = volts%100;  // extract decimals
                volts = volts/100;      //extract integer volts
                Lcd_Set_Cursor(2,1);	// select line 2
                //  sprintf(outString,"ANO: %3.3Xh, %u.%2.2uV",adres,volts,decivolts);
                sprintf(outString,"V- PSU = -%u.%2.2uV",volts,decivolts);
                Lcd_Write_String(outString);
            }
                //    last_value = adres;
                DelayMs(300);
            }
    }
//************************************************************************
//******************* END OF ANALOG INPUTS *******************************
    
// *********** MA1N SCREEN DISPLAY WITH DELAYS BETWEEN MESSAGES **********
        else if (HADFAULT == 1)  // DISPLAY THE TITLE SCREEN WHILE NO FAULT
        {
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position
            Lcd_Write_String(MSG_T1A);
            Lcd_Set_Cursor(2,1);      // Select second line, first char poisition
            Lcd_Write_String(MSG_T1B);
            DelayMs(1000);            //DELAY APPROX 1 SECOND

            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char position
            Lcd_Write_String(MSG_T2A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String(MSG_T2B);
            DelayMs(1000);             //DELAY APPROX 1 SECOND

            //   THE MAIN DISPLAY.
            Lcd_Init();               // initialize LCD
            Lcd_Clear();              // clear LCD
            Lcd_Set_Cursor(1,1);;     // select first line first char positione
            Lcd_Write_String(MSG_T3A);
            Lcd_Set_Cursor(2,1);   // Select second line, first char poisition
            Lcd_Write_String(MSG_T3B);
            HADFAULT = 0 ;
 /*
            // SCROLL LEFT THEN RIGHT           
            for(a=0;a<15;a++)
            {
                DelayMs(500);
                Lcd_Shift_Left();
            }
            for(a=0;a<15;a++)
            {
                DelayMs(500);
                Lcd_Shift_Right();
            }
*/

        }
    }
}
 
